#!/bin/bash
#
# SPDX-FileCopyrightText: 2023 David Runge <dave@sleepmap.de>
# SPDX-License-Identifier: Apache-2.0 OR MIT

set -euxo pipefail

readonly target="${1:-}"

run_fmt() {
  cargo --version
  rustc --version
  cargo +nightly fmt -- --check
}

run_clippy() {
  cargo --version
  rustc --version
  cargo clippy --no-deps
}

run_commits() {
  local known_good="2e1f163c68a47800574d874ecd9f6a97de637675"
  # NOTE: check only from first known good version onwards
  cog check "$known_good"..HEAD
}

run_deny() {
  cargo --version
  rustc --version
  cargo deny check
}

run_codespell() {
  codespell
}

run_integration() {
  local path
  path="$(dirname "$(realpath "$0")")"

  if [[ -z "$(podman images -n -f "reference=oct")" ]]; then
    podman image build "$path/../" -t oct --squash
  fi
  podman container run --mount=type=bind,source="$path/../",destination=/src oct
}

run_msrv() {
  cargo --version
  rustc --version
  cargo msrv list
  cargo +nightly check -Zdirect-minimal-versions
}

run_reuse() {
  reuse lint
}

run_test() {
  cargo --version
  rustc --version
  cargo test --all
}

run_udeps() {
  cargo --version
  rustc --version
  cargo +nightly udeps --all-features --all-targets
}

if [[ -n "$target" ]]; then
  case "$target" in
    clippy)
      run_clippy
    ;;
    codespell)
      run_codespell
    ;;
    commits)
      run_commits
    ;;
    deny)
      run_deny
    ;;
    fmt)
      run_fmt
    ;;
    integration)
      run_integration
    ;;
    msrv)
      run_msrv
    ;;
    reuse)
      run_reuse
    ;;
    test)
      run_test
    ;;
    udeps)
      run_udeps
    ;;
  esac
else
  run_reuse
  run_codespell
  run_fmt
  run_clippy
  run_udeps
  run_test
  run_msrv
  run_deny
fi
